@@ -17,12 +17,15 @@
  */
 
 #include <math.h>
+#include <common/geomprocs.h>
 #include <common/render.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
+#include <util/gv_math.h>
+#include <util/streq.h>
 #include <util/unreachable.h>
 
 #ifdef DEBUG
@@ -39,19 +42,18 @@ static void showPoints(pointf ps[], int pn)
 {
     int bi;
 
-    show_boxes_append(&Show_boxes, gv_strdup("%% self list"));
-    show_boxes_append(&Show_boxes, gv_strdup("dbgstart"));
+    LIST_APPEND(&Show_boxes, gv_strdup("%% self list"));
+    LIST_APPEND(&Show_boxes, gv_strdup("dbgstart"));
     for (bi = 0; bi < pn; bi++) {
 	agxbuf buf = {0};
 	agxbprint(&buf, "%.5g %.5g point", ps[bi].x, ps[bi].y);
-	show_boxes_append(&Show_boxes, agxbdisown(&buf));
+	LIST_APPEND(&Show_boxes, agxbdisown(&buf));
     }
-    show_boxes_append(&Show_boxes, gv_strdup("grestore"));
+    LIST_APPEND(&Show_boxes, gv_strdup("grestore"));
 }
 #endif
 
-/* arrow_clip:
- * Clip arrow to node boundary.
+/* Clip arrow to node boundary.
  * The real work is done elsewhere. Here we get the real edge,
  * check that the edge has arrowheads, and that an endpoint
  * isn't a merge point where several parts of an edge meet.
@@ -95,8 +97,7 @@ arrow_clip(edge_t * fe, node_t * hn,
     }
 }
 
-/* bezier_clip
- * Clip bezier to shape using binary search.
+/* Clip Bézier to shape using binary search.
  * The details of the shape are passed in the inside_context;
  * The function providing the inside test is passed as a parameter.
  * left_inside specifies that sp[0] is inside the node,
@@ -149,8 +150,7 @@ void bezier_clip(inside_t * inside_context,
 	    sp[i] = seg[i];
 }
 
-/* shape_clip0:
- * Clip Bezier to node shape using binary search.
+/* Clip Bézier to node shape using binary search.
  * left_inside specifies that curve[0] is inside the node, else
  * curve[3] is taken as inside.
  * Assumes ND_shape(n) and ND_shape(n)->fns->insidefn are non-NULL.
@@ -179,12 +179,11 @@ shape_clip0(inside_t * inside_context, node_t * n, pointf curve[4],
     ND_rw(n) = save_real_size;
 }
 
-/* shape_clip:
- * Clip Bezier to node shape
+/* Clip Bézier to node shape
  * Uses curve[0] to determine which which side is inside the node.
  * NOTE: This test is bad. It is possible for previous call to
- * shape_clip to produce a Bezier with curve[0] moved to the boundary
- * for which insidefn(curve[0]) is true. Thus, if the new Bezier is
+ * shape_clip to produce a Bézier with curve[0] moved to the boundary
+ * for which insidefn(curve[0]) is true. Thus, if the new Bézier is
  * fed back to shape_clip, it will again assume left_inside is true.
  * To be safe, shape_clip0 should guarantee that the computed boundary
  * point fails insidefn.
@@ -209,9 +208,7 @@ void shape_clip(node_t * n, pointf curve[4])
     shape_clip0(&inside_context, n, curve, left_inside);
 }
 
-/* new_spline:
- * Create and attach a new bezier of size sz to the edge d
- */
+/// create and attach a new Bézier of size sz to the edge d
 bezier *new_spline(edge_t *e, size_t sz) {
     bezier *rv;
     while (ED_to_orig(e) != NULL && ED_edge_type(e) != NORMAL)
@@ -228,8 +225,7 @@ bezier *new_spline(edge_t *e, size_t sz) {
     return rv;
 }
 
-/* clip_and_install:
- * Given a raw spline (pn control points in ps), representing
+/* Given a raw spline (pn control points in ps), representing
  * a path from edge agtail(fe) ending in node hn, clip the ends to
  * the node boundaries and attach the resulting spline to the
  * edge.
@@ -238,28 +234,21 @@ void
 clip_and_install(edge_t *fe, node_t *hn, pointf *ps, size_t pn,
 		 splineInfo * info)
 {
-    pointf p2;
-    bezier *newspl;
-    node_t *tn;
     int clipTail, clipHead;
     size_t start, end;
-    graph_t *g;
     edge_t *orig;
     boxf *tbox, *hbox;
 
-    tn = agtail(fe);
-    g = agraphof(tn);
-    newspl = new_spline(fe, pn);
+    node_t *tn = agtail(fe);
+    graph_t *const g = agraphof(tn);
+    bezier *const newspl = new_spline(fe, pn);
 
     for (orig = fe; ED_to_orig(orig) != NULL && ED_edge_type(orig) != NORMAL;
          orig = ED_to_orig(orig));
 
     /* may be a reversed flat edge */
     if (!info->ignoreSwap && ND_rank(tn) == ND_rank(hn) && ND_order(tn) > ND_order(hn)) {
-	node_t *tmp;
-	tmp = hn;
-	hn = tn;
-	tn = tmp;
+	SWAP(&hn, &tn);
     }
     if (tn == agtail(orig)) {
 	clipTail = ED_tail_port(orig).clip;
@@ -278,8 +267,8 @@ clip_and_install(edge_t *fe, node_t *hn, pointf *ps, size_t pn,
     if(clipTail && ND_shape(tn) && ND_shape(tn)->fns->insidefn) {
 	inside_t inside_context = {.s = {.n = tn, .bp = tbox}};
 	for (start = 0; start < pn - 4; start += 3) {
-	    p2.x = ps[start + 3].x - ND_coord(tn).x;
-	    p2.y = ps[start + 3].y - ND_coord(tn).y;
+	    const pointf p2 = {.x = ps[start + 3].x - ND_coord(tn).x,
+	                       .y = ps[start + 3].y - ND_coord(tn).y};
 	    if (!ND_shape(tn)->fns->insidefn(&inside_context, p2))
 		break;
 	}
@@ -289,8 +278,8 @@ clip_and_install(edge_t *fe, node_t *hn, pointf *ps, size_t pn,
     if(clipHead && ND_shape(hn) && ND_shape(hn)->fns->insidefn) {
 	inside_t inside_context = {.s = {.n = hn, .bp = hbox}};
 	for (end = pn - 4; end > 0; end -= 3) {
-	    p2.x = ps[end].x - ND_coord(hn).x;
-	    p2.y = ps[end].y - ND_coord(hn).y;
+	    const pointf p2 = {.x = ps[end].x - ND_coord(hn).x,
+	                       .y = ps[end].y - ND_coord(hn).y};
 	    if (!ND_shape(hn)->fns->insidefn(&inside_context, p2))
 		break;
 	}
@@ -328,7 +317,6 @@ conc_slope(node_t* n)
 {
     double s_in, s_out, m_in, m_out;
     int cnt_in, cnt_out;
-    pointf p;
     edge_t *e;
 
     s_in = s_out = 0.0;
@@ -336,12 +324,12 @@ conc_slope(node_t* n)
 	s_in += ND_coord(agtail(e)).x;
     for (cnt_out = 0; (e = ND_out(n).list[cnt_out]); cnt_out++)
 	s_out += ND_coord(aghead(e)).x;
-    p.x = ND_coord(n).x - (s_in / cnt_in);
-    p.y = ND_coord(n).y - ND_coord(agtail(ND_in(n).list[0])).y;
-    m_in = atan2(p.y, p.x);
-    p.x = s_out / cnt_out - ND_coord(n).x;
-    p.y = ND_coord(aghead(ND_out(n).list[0])).y - ND_coord(n).y;
-    m_out = atan2(p.y, p.x);
+    const double x1 = ND_coord(n).x - s_in / cnt_in;
+    const double y1 = ND_coord(n).y - ND_coord(agtail(ND_in(n).list[0])).y;
+    m_in = atan2(y1, x1);
+    const double x2 = s_out / cnt_out - ND_coord(n).x;
+    const double y2 = ND_coord(aghead(ND_out(n).list[0])).y - ND_coord(n).y;
+    m_out = atan2(y2, x2);
     return (m_in + m_out) / 2.0;
 }
 
@@ -351,8 +339,7 @@ void add_box(path * P, boxf b)
 	P->boxes[P->nbox++] = b;
 }
 
-/* beginpath:
- * Set up boxes near the tail node.
+/* Set up boxes near the tail node.
  * For regular nodes, the result should be a list of contiguous rectangles
  * such that the last one has the smallest LL.y and its LL.y is above
  * the bottom of the rank (rank.ht1).
@@ -824,7 +811,6 @@ static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     edge_t *e;
     int sgn, point_pair;
     double hy, ty, stepx, dx, dy, height;
-    pointf points[1000];
 
     e = edges[ind];
     n = agtail(e);
@@ -858,14 +844,15 @@ static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
         ty += stepy;
         hy += stepy;
         dx += sgn * stepx;
-        size_t pointn = 0;
-        points[pointn++] = tp;
-        points[pointn++] = (pointf){tp.x + dx, tp.y - ty / 3};
-        points[pointn++] = (pointf){tp.x + dx, np.y - dy};
-        points[pointn++] = (pointf){(tp.x + hp.x) / 2, np.y - dy};
-        points[pointn++] = (pointf){hp.x - dx, np.y - dy};
-        points[pointn++] = (pointf){hp.x - dx, hp.y - hy / 3};
-        points[pointn++] = hp;
+        pointf points[] = {
+    	    tp,
+    	    {tp.x + dx, tp.y - ty / 3},
+    	    {tp.x + dx, np.y - dy},
+    	    {(tp.x + hp.x) / 2, np.y - dy},
+    	    {hp.x - dx, np.y - dy},
+    	    {hp.x - dx, hp.y - hy / 3},
+    	    hp
+        };
         if (ED_label(e)) {
 	if (GD_flip(agraphof(agtail(e)))) {
     	    height = ED_label(e)->dimen.x;
@@ -878,6 +865,7 @@ static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     	if (height > stepy)
     	    dy += height - stepy;
         }
+        const size_t pointn = sizeof(points) / sizeof(points[0]);
         clip_and_install(e, aghead(e), points, pointn, sinfo);
 #ifdef DEBUG
         if (debugleveln(e,1))
@@ -893,7 +881,6 @@ static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
-    pointf points[1000];
 
     e = edges[ind];
     n = agtail(e);
@@ -964,14 +951,15 @@ static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
         ty += stepy;
         hy += stepy;
         dx += sgn * stepx;
-        size_t pointn = 0;
-        points[pointn++] = tp;
-        points[pointn++] = (pointf){tp.x + dx, tp.y + ty / 3};
-        points[pointn++] = (pointf){tp.x + dx, np.y + dy};
-        points[pointn++] = (pointf){(tp.x + hp.x) / 2, np.y + dy};
-        points[pointn++] = (pointf){hp.x - dx, np.y + dy};
-        points[pointn++] = (pointf){hp.x - dx, hp.y + hy / 3};
-        points[pointn++] = hp;
+        pointf points[] = {
+	    tp,
+	    {tp.x + dx, tp.y + ty / 3},
+	    {tp.x + dx, np.y + dy},
+	    {(tp.x + hp.x) / 2, np.y + dy},
+	    {hp.x - dx, np.y + dy},
+	    {hp.x - dx, hp.y + hy / 3},
+	    hp,
+        };
         if (ED_label(e)) {
 	    if (GD_flip(agraphof(agtail(e)))) {
 		height = ED_label(e)->dimen.x;
@@ -984,6 +972,7 @@ static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
 	    if (height > stepy)
 		dy += height - stepy;
         }
+        const size_t pointn = sizeof(points) / sizeof(points[0]);
        clip_and_install(e, aghead(e), points, pointn, sinfo);
 #ifdef DEBUG
         if (debugleveln(e,1))
@@ -999,7 +988,6 @@ static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
-    pointf points[1000];
 
     e = edges[ind];
     n = agtail(e);
@@ -1034,14 +1022,15 @@ static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
         tx += stepx;
         hx += stepx;
         dy += sgn * stepy;
-        size_t pointn = 0;
-        points[pointn++] = tp;
-        points[pointn++] = (pointf){tp.x + tx / 3, tp.y + dy};
-        points[pointn++] = (pointf){np.x + dx, tp.y + dy};
-        points[pointn++] = (pointf){np.x + dx, (tp.y + hp.y) / 2};
-        points[pointn++] = (pointf){np.x + dx, hp.y - dy};
-        points[pointn++] = (pointf){hp.x + hx / 3, hp.y - dy};
-        points[pointn++] = hp;
+        pointf points[] = {
+	    tp,
+	    {tp.x + tx / 3, tp.y + dy},
+	    {np.x + dx, tp.y + dy},
+	    {np.x + dx, (tp.y + hp.y) / 2},
+	    {np.x + dx, hp.y - dy},
+	    {hp.x + hx / 3, hp.y - dy},
+	    hp,
+        };
         if (ED_label(e)) {
 	    if (GD_flip(agraphof(agtail(e)))) {
 		width = ED_label(e)->dimen.y;
@@ -1054,6 +1043,7 @@ static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
 	    if (width > stepx)
 		dx += width - stepx;
         }
+        const size_t pointn = sizeof(points) / sizeof(points[0]);
 	clip_and_install(e, aghead(e), points, pointn, sinfo);
 #ifdef DEBUG
         if (debugleveln(e,1))
@@ -1069,7 +1059,6 @@ static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
-    pointf points[1000];
 
     e = edges[ind];
     n = agtail(e);
@@ -1107,15 +1096,15 @@ static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
         tx += stepx;
         hx += stepx;
         dy += sgn * stepy;
-        size_t pointn = 0;
-        points[pointn++] = tp;
-        points[pointn++] = (pointf){tp.x - tx / 3, tp.y + dy};
-        points[pointn++] = (pointf){np.x - dx, tp.y + dy};
-        points[pointn++] = (pointf){np.x - dx, (tp.y + hp.y) / 2};
-        points[pointn++] = (pointf){np.x - dx, hp.y - dy};
-        points[pointn++] = (pointf){hp.x - hx / 3, hp.y - dy};
-
-        points[pointn++] = hp;
+        pointf points[] = {
+    	    tp,
+    	    {tp.x - tx / 3, tp.y + dy},
+    	    {np.x - dx, tp.y + dy},
+    	    {np.x - dx, (tp.y + hp.y) / 2},
+    	    {np.x - dx, hp.y - dy},
+    	    {hp.x - hx / 3, hp.y - dy},
+    	    hp,
+        };
         if (ED_label(e)) {
     	if (GD_flip(agraphof(agtail(e)))) {
     	    width = ED_label(e)->dimen.y;
@@ -1129,6 +1118,7 @@ static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     	    dx += width - stepx;
         }
 
+        const size_t pointn = sizeof(points) / sizeof(points[0]);
         clip_and_install(e, aghead(e), points, pointn, sinfo);
 #ifdef DEBUG
         if (debugleveln(e,1))
@@ -1137,8 +1127,7 @@ static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
     }
 }
 
-/* selfRightSpace:
- * Assume e is self-edge.
+/* Assume e is self-edge.
  * Return extra space necessary on the right for this edge.
  * If the edge does not go on the right, return 0.
  * NOTE: the actual space is determined dynamically by GD_nodesep,
@@ -1165,8 +1154,7 @@ double selfRightSpace(edge_t *e) {
     return sw;
 }
 
-/* makeSelfEdge:
- * The routing is biased towards the right side because this is what
+/* The routing is biased towards the right side because this is what
  * dot supports, and leaves room for.
  * FIX: With this bias, labels tend to be placed on top of each other.
  * Perhaps for self-edges, the label should be centered.
@@ -1211,9 +1199,7 @@ void makeSelfEdge(edge_t *edges[], size_t ind, size_t cnt, double sizex,
     else assert(0);
 }
 
-/* makePortLabels:
- * Add head and tail labels if necessary and update bounding box.
- */
+/// add head and tail labels if necessary and update bounding box
 void makePortLabels(edge_t * e)
 {
     /* Only use this if labelangle or labeldistance is set for the edge;
@@ -1231,10 +1217,7 @@ void makePortLabels(edge_t * e)
     }
 }
 
-/* endPoints:
- * Extract the actual end points of the spline, where
- * they touch the node.
- */
+/// extract the actual end points of the spline, where they touch the node
 static void endPoints(splines * spl, pointf * p, pointf * q)
 {
     bezier bz;
@@ -1255,8 +1238,7 @@ static void endPoints(splines * spl, pointf * p, pointf * q)
     }
 }
 
-/* polylineMidpoint;
- * Find midpoint of polyline.
+/* Find midpoint of polyline.
  * pp and pq are set to the endpoints of the line segment containing it.
  */
 static pointf
@@ -1299,15 +1281,14 @@ pointf
 edgeMidpoint (graph_t* g, edge_t * e)
 {
     int et = EDGE_TYPE (g);
-    pointf d, spf, p, q;
+    pointf spf, p, q;
 
     endPoints(ED_spl(e), &p, &q);
     if (APPROXEQPT(p, q, MILLIPOINT)) { /* degenerate spline */
 	spf = p;
     }
     else if (et == EDGETYPE_SPLINE || et == EDGETYPE_CURVED) {
-	d.x = (q.x + p.x) / 2.;
-	d.y = (p.y + q.y) / 2.;
+	const pointf d = mid_pointf(p, q);
 	spf = dotneato_closest(ED_spl(e), d);
     }
     else {   /* EDGETYPE_PLINE, EDGETYPE_ORTHO or EDGETYPE_LINE */
@@ -1317,11 +1298,7 @@ edgeMidpoint (graph_t* g, edge_t * e)
     return spf;
 }
 
-#define LEFTOF(a,b,c) (((a.y - b.y)*(c.x - b.x) - (c.y - b.y)*(a.x - b.x)) > 0)
-#define MAXLABELWD  (POINTS_PER_INCH/2.0)
-
-/* addEdgeLabels:
- * Adds label, headlabel and taillabel.
+/* Adds label, headlabel and taillabel.
  * Updates bounding box.
  * We use the endpoints of the spline.
  */
@@ -1329,60 +1306,52 @@ void addEdgeLabels(edge_t *e) {
     makePortLabels(e);
 }
 
-#define AGXGET(o,a) agxget(o,a)
-
-/* place_portlabel:
- * place the {head,tail}label (depending on HEAD_P) of edge E
+/* place the {head,tail}label (depending on HEAD_P) of edge E
  * N.B. Assume edges are normalized, so tail is at spl->list[0].list[0]
  * and head is at spl->list[spl->size-l].list[bez->size-1]
  * Return 1 on success
  */
 int place_portlabel(edge_t * e, bool head_p)
 {
-    textlabel_t *l;
     splines *spl;
-    bezier *bez;
-    double dist, angle;
-    pointf c[4], pe, pf;
-    char* la;
-    char* ld;
+    pointf pe, pf;
 
     if (ED_edge_type(e) == IGNORED)
 	return 0;
     /* add label here only if labelangle or labeldistance is defined; else, use external label */
-    if ((!E_labelangle || *(la = AGXGET(e,E_labelangle)) == '\0') &&
-	(!E_labeldistance || *(ld = AGXGET(e,E_labeldistance)) == '\0')) {
+    if ((!E_labelangle || streq(agxget(e, E_labelangle), "")) &&
+	(!E_labeldistance || streq(agxget(e, E_labeldistance), ""))) {
 	return 0;
     }
 
-    l = head_p ? ED_head_label(e) : ED_tail_label(e);
+    textlabel_t *const l = head_p ? ED_head_label(e) : ED_tail_label(e);
     if ((spl = getsplinepoints(e)) == NULL) return 0;
     if (!head_p) {
-	bez = &spl->list[0];
+	const bezier *const bez = &spl->list[0];
 	if (bez->sflag) {
 	    pe = bez->sp;
 	    pf = bez->list[0];
 	} else {
 	    pe = bez->list[0];
-	    for (size_t i = 0; i < 4; i++)
-		c[i] = bez->list[i];
+	    const pointf *const c = bez->list; // slice of the first 4 points
 	    pf = Bezier(c, 0.1, NULL, NULL);
 	}
     } else {
-	bez = &spl->list[spl->size - 1];
+	const bezier *const bez = &spl->list[spl->size - 1];
 	if (bez->eflag) {
 	    pe = bez->ep;
 	    pf = bez->list[bez->size - 1];
 	} else {
 	    pe = bez->list[bez->size - 1];
-	    for (size_t i = 0; i < 4; i++)
-		c[i] = bez->list[bez->size - 4 + i];
+	    // slice of the last 4 points
+	    const pointf *const c = &bez->list[bez->size - 4];
 	    pf = Bezier(c, 0.9, NULL, NULL);
 	}
     }
-    angle = atan2(pf.y - pe.y, pf.x - pe.x) +
+    const double angle = atan2(pf.y - pe.y, pf.x - pe.x) +
 	RADIANS(late_double(e, E_labelangle, PORT_LABEL_ANGLE, -180.0));
-    dist = PORT_LABEL_DISTANCE * late_double(e, E_labeldistance, 1.0, 0.0);
+    const double dist =
+      PORT_LABEL_DISTANCE * late_double(e, E_labeldistance, 1.0, 0.0);
     l->pos.x = pe.x + dist * cos(angle);
     l->pos.y = pe.y + dist * sin(angle);
     l->set = true;
