@@ -3,15 +3,14 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-
+#include <assert.h>
 #include "config.h"
-#include <cgraph/alloc.h>
-#include <cgraph/unreachable.h>
+#include <limits.h>
 #include <math.h>
 #include <neatogen/neato.h>
 #include <neatogen/adjust.h>
@@ -19,6 +18,10 @@
 #include <pathplan/vispath.h>
 #include <neatogen/multispline.h>
 #include <stdbool.h>
+#include <stddef.h>
+#include <util/alloc.h>
+#include <util/gv_math.h>
+#include <util/unreachable.h>
 
 #ifdef ORTHO
 #include <ortho/ortho.h>
@@ -40,13 +43,11 @@ static bool swap_ends_p(edge_t * e)
 static splineInfo sinfo = {.swapEnds = swap_ends_p,
                            .splineMerge = spline_merge};
 
-static void
-make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
-	      Pedge_t ** barriers, int *n_barriers)
-{
-    int i, j, k, n, b;
+static void make_barriers(Ppoly_t **poly, int npoly, int pp, int qp,
+                          Pedge_t **barriers, size_t *n_barriers) {
+    int i, j, k;
 
-    n = 0;
+    size_t n = 0;
     for (i = 0; i < npoly; i++) {
 	if (i == pp)
 	    continue;
@@ -55,15 +56,15 @@ make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
 	n += poly[i]->pn;
     }
     Pedge_t *bar = gv_calloc(n, sizeof(Pedge_t));
-    b = 0;
+    size_t b = 0;
     for (i = 0; i < npoly; i++) {
 	if (i == pp)
 	    continue;
 	if (i == qp)
 	    continue;
-	for (j = 0; j < poly[i]->pn; j++) {
+	for (j = 0; j < (int)poly[i]->pn; j++) {
 	    k = j + 1;
-	    if (k >= poly[i]->pn)
+	    if (k >= (int)poly[i]->pn)
 		k = 0;
 	    bar[b].a = poly[i]->ps[j];
 	    bar[b].b = poly[i]->ps[k];
@@ -105,7 +106,8 @@ typedef struct {
     edge_t *e;
 } edgeitem;
 
-static void *newitem(edgeitem *obj, Dtdisc_t *disc) {
+static void *newitem(void *p, Dtdisc_t *disc) {
+    edgeitem *obj = p;
     edgeitem *newp;
 
     (void)disc;
@@ -117,17 +119,9 @@ static void *newitem(edgeitem *obj, Dtdisc_t *disc) {
     return newp;
 }
 
-static void freeitem(edgeitem *obj, Dtdisc_t *disc) {
-    (void)disc;
-    free(obj);
-}
-
-static int
-cmpitems(Dt_t * d, edgeinfo * key1, edgeinfo * key2, Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-
+static int cmpitems(void *k1, void *k2) {
+    const edgeinfo *key1 = k1;
+    const edgeinfo *key2 = k2;
     if (key1->n1 > key2->n1)
 	return 1;
     if (key1->n1 < key2->n1)
@@ -160,14 +154,12 @@ Dtdisc_t edgeItemDisc = {
     offsetof(edgeitem, id),
     sizeof(edgeinfo),
     offsetof(edgeitem, link),
-    (Dtmake_f) newitem,
-    (Dtfree_f) freeitem,
-    (Dtcompar_f) cmpitems,
-    0,
+    newitem,
+    free,
+    cmpitems,
 };
 
-/* equivEdge:
- * See if we have already encountered an edge between the same
+/* See if we have already encountered an edge between the same
  * node:port pairs. If so, return the earlier edge. If not, 
  * this edge is added to map and returned.
  * We first have to canonicalize the key fields using a lexicographic
@@ -216,15 +208,15 @@ static edge_t *equivEdge(Dt_t * map, edge_t * e)
 }
 
 
-/* makeSelfArcs:
- * Generate loops. We use the library routine makeSelfEdge
+/* Generate loops. We use the library routine makeSelfEdge
  * which also places the labels.
  * We have to handle port labels here.
  * as well as update the bbox from edge labels.
  */
 void makeSelfArcs(edge_t * e, int stepx)
 {
-    int cnt = ED_count(e);
+    assert(ED_count(e) >= 0);
+    const size_t cnt = (size_t)ED_count(e);
 
     if (cnt == 1 || Concentrate) {
 	edge_t *edges1[1];
@@ -233,15 +225,14 @@ void makeSelfArcs(edge_t * e, int stepx)
 	if (ED_label(e))
 	    updateBB(agraphof(agtail(e)), ED_label(e));
 	makePortLabels(e);
-    } else {
-	int i;
+    } else if (cnt > 1) {
 	edge_t **edges = gv_calloc(cnt, sizeof(edge_t*));
-	for (i = 0; i < cnt; i++) {
+	for (size_t i = 0; i < cnt; i++) {
 	    edges[i] = e;
 	    e = ED_to_virt(e);
 	}
 	makeSelfEdge(edges, 0, cnt, stepx, stepx, &sinfo);
-	for (i = 0; i < cnt; i++) {
+	for (size_t i = 0; i < cnt; i++) {
 	    e = edges[i];
 	    if (ED_label(e))
 		updateBB(agraphof(agtail(e)), ED_label(e));
@@ -251,8 +242,75 @@ void makeSelfArcs(edge_t * e, int stepx)
     }
 }
 
-/* makeObstacle:
- * Given a node, return an obstacle reflecting the
+/** calculate the slope of the tangent of an ellipse
+ *
+ * The equation for the slope `m` of the tangent of an ellipse as a function of
+ * `x` * is given by:
+ *
+ *           bx
+ * m = ± ――――――――――
+ *          _______
+ *       a √ a²- x²
+ *
+ * or
+ *
+ * m = ± (b * x) / (a * sqrt(a * a - x * x))
+ *
+ * We know that the slope is negative in the first and third quadrant, i.e.,
+ * when the signs of x and y are the same, so we use that to select the correct
+ * slope.
+ *
+ * @param a Half the width of the ellipse, i.e., the maximum x value
+ * @param b Half the height of the ellipse, i.e., the maximum y value
+ * @param p A point on the ellipse periphery in which to calculate the slope of
+ * the tangent
+ * @return The slope of the tangent in point `p`
+ */
+static double ellipse_tangent_slope(double a, double b, pointf p) {
+  assert(p.x != a &&
+         "cannot handle ellipse tangent slope in horizontal extreme point");
+  const double sign_y = p.y >= 0 ? 1 : -1;
+  const double m = -sign_y * (b * p.x) / (a * sqrt(a * a - p.x * p.x));
+  assert(isfinite(m) && "ellipse tangent slope is infinite");
+  return m;
+}
+
+/** calculate the intersection of two lines
+ *
+ * @param l0 First line
+ * @param l1 Second line
+ * @return Intersection of the two lines
+ */
+static pointf line_intersection(linef l0, linef l1) {
+  const double x =
+      (l0.m * l0.p.x - l0.p.y - l1.m * l1.p.x + l1.p.y) / (l0.m - l1.m);
+  const double y = l0.p.y + l0.m * (x - l0.p.x);
+  return (pointf){x, y};
+}
+
+/** calculate a corner of a polygon circumscribed about an ellipse
+ *
+ * @param a Half the width of the ellipse, i.e., the maximum x value
+ * @param b Half the height of the ellipse, i.e., the maximum y value
+ * @param i Index of the polygon corner
+ * @param nsides Number of sides of the polygon
+ * @return Polygon corner at index `i`
+ */
+static pointf circumscribed_polygon_corner_about_ellipse(double a, double b,
+                                                         size_t i,
+                                                         size_t nsides) {
+  const double angle0 = 2.0 * M_PI * ((double)i - 0.5) / (double)nsides;
+  const double angle1 = 2.0 * M_PI * ((double)i + 0.5) / (double)nsides;
+  const pointf p0 = {a * cos(angle0), b * sin(angle0)};
+  const pointf p1 = {a * cos(angle1), b * sin(angle1)};
+  const double m0 = ellipse_tangent_slope(a, b, p0);
+  const double m1 = ellipse_tangent_slope(a, b, p1);
+  const linef line0 = {{p0.x, p0.y}, m0};
+  const linef line1 = {{p1.x, p1.y}, m1};
+  return line_intersection(line0, line1);
+}
+
+/* Given a node, return an obstacle reflecting the
  * node's geometry. pmargin specifies how much space to allow
  * around the polygon. 
  * Returns the constructed polygon on success, NULL on failure.
@@ -267,8 +325,7 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 {
     Ppoly_t *obs;
     polygon_t *poly;
-    double adj = 0.0;
-    int j, sides;
+    size_t sides;
     pointf polyp;
     boxf b;
     pointf pt;
@@ -291,7 +348,7 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 		/* For fixedshape, we can't use the width and height, as this includes
 		 * the label. We only want to use the actual node shape.
 		 */
-	    if (poly->option & FIXEDSHAPE) {
+	    if (poly->option.fixedshape) {
 		b = polyBB (poly);
 		vs[0] = b.LL;
 		vs[1].x = b.UR.x;
@@ -300,33 +357,43 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 		vs[3].x = b.LL.x;
 		vs[3].y = b.UR.y;
 	    } else {
-		p.x = -ND_lw(n);
-		p.y = -ND_ht(n)/2.0;
+		const double width = ND_lw(n) + ND_rw(n);
+		const double outline_width = INCH2PS(ND_outline_width(n));
+		// scale lw and rw proportionally to sum to outline width
+		const double outline_lw = ND_lw(n) * outline_width / width;
+		const double outline_ht = INCH2PS(ND_outline_height(n));
+		p.x = -outline_lw;
+		p.y = -outline_ht / 2.0;
 		vs[0] = p;
-		p.x = ND_lw(n);
+		p.x = outline_lw;
 		vs[1] = p;
-		p.y = ND_ht(n)/2.0;
+		p.y = outline_ht / 2.0;
 		vs[2] = p;
-		p.x = -ND_lw(n);
+		p.x = -outline_lw;
 		vs[3] = p;
 	    }
 	}
 	else if (poly->sides >= 3) {
 	    isPoly = true;
 	    sides = poly->sides;
-	    verts = poly->vertices;
-	    margin.x = pmargin->x;
-	    margin.y = pmargin->y;
+            const double penwidth = late_double(n, N_penwidth, 1.0, 0.0);
+            // possibly use extra vertices representing the outline, i.e., the
+            // outermost periphery with penwidth taken into account
+            const size_t extra_peripheries = poly->peripheries >= 1 && penwidth > 0.0 ? 1 : 0;
+            const size_t outline_periphery = poly->peripheries + extra_peripheries;
+            const size_t vertices_offset = outline_periphery >= 1 ? (outline_periphery - 1) * sides : 0;
+            verts = poly->vertices + vertices_offset;
+            margin.x = pmargin->x;
+            margin.y = pmargin->y;
 	} else {		/* ellipse */
 	    isPoly = false;
 	    sides = 8;
-	    adj = drand48() * .01;
 	}
 	obs->pn = sides;
 	obs->ps = gv_calloc(sides, sizeof(Ppoint_t));
 	/* assuming polys are in CCW order, and pathplan needs CW */
-	for (j = 0; j < sides; j++) {
-	    double xmargin = 0.0, ymargin = 0.0;
+	for (size_t j = 0; j < sides; j++) {
+	    double xmargin = 0, ymargin = 0;
 	    if (isPoly) {
 		if (pmargin->doAdd) {
 		    if (sides == 4) {
@@ -354,7 +421,7 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 			polyp.y = verts[j].y + ymargin;
 		    }
 		    else {
-			double h = LEN(verts[j].x,verts[j].y);
+			const double h = hypot(verts[j].x, verts[j].y);
 			polyp.x = verts[j].x * (1.0 + margin.x/h);
 			polyp.y = verts[j].y * (1.0 + margin.y/h);
 		    }
@@ -364,17 +431,12 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 		    polyp.y = verts[j].y * margin.y;
 		}
 	    } else {
-		double c, s;
-		c = cos(2.0 * M_PI * j / sides + adj);
-		s = sin(2.0 * M_PI * j / sides + adj);
-		if (pmargin->doAdd) {
-		    polyp.x =  c*(ND_lw(n)+ND_rw(n)+pmargin->x) / 2.0;
-		    polyp.y =  s*(ND_ht(n)+pmargin->y) / 2.0;
-		}
-		else {
-		    polyp.x = pmargin->x * c * (ND_lw(n) + ND_rw(n)) / 2.0;
-		    polyp.y = pmargin->y * s * ND_ht(n) / 2.0;
-		}
+		const double width = INCH2PS(ND_outline_width(n));
+		const double height = INCH2PS(ND_outline_height(n));
+		margin = pmargin->doAdd ? (pointf){pmargin->x, pmargin->y} : (pointf){0};
+		const double ellipse_a = (width + margin.x) / 2.0;
+		const double ellipse_b = (height + margin.y) / 2.0;
+		polyp = circumscribed_polygon_corner_about_ellipse(ellipse_a, ellipse_b, j, sides);
 	    }
 	    obs->ps[sides - j - 1].x = polyp.x + ND_coord(n).x;
 	    obs->ps[sides - j - 1].y = polyp.y + ND_coord(n).y;
@@ -427,8 +489,7 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
     return obs;
 }
 
-/* getPath
- * Construct the shortest path from one endpoint of e to the other.
+/* Construct the shortest path from one endpoint of e to the other.
  * vconfig is a precomputed data structure to help in the computation.
  * If chkPts is true, the function finds the polygons, if any, containing
  * the endpoints and tells the shortest path computation to ignore them. 
@@ -463,8 +524,7 @@ static void makePolyline(edge_t * e) {
     addEdgeLabels(e);
 }
 
-/* makeSpline:
- * Construct a spline connecting the endpoints of e, avoiding the npoly
+/* Construct a spline connecting the endpoints of e, avoiding the npoly
  * obstacles obs.
  * The resultant spline is attached to the edge, the positions of any 
  * edge labels are computed, and the graph's bounding box is recomputed.
@@ -475,8 +535,7 @@ static void makePolyline(edge_t * e) {
  */
 void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
     Ppolyline_t line, spline;
-    Pvector_t slopes[2];
-    int i, n_barriers;
+    int i;
     int pp, qp;
     Ppoint_t p, q;
     Pedge_t *barriers;
@@ -494,11 +553,11 @@ void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
 		qp = i;
 	}
 
+    size_t n_barriers;
     make_barriers(obs, npoly, pp, qp, &barriers, &n_barriers);
-    slopes[0].x = slopes[0].y = 0.0;
-    slopes[1].x = slopes[1].y = 0.0;
+    Pvector_t slopes[2] = {0};
     if (Proutespline(barriers, n_barriers, line, slopes, &spline) < 0) {
-	agerr (AGERR, "makeSpline: failed to make spline edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+	agerrorf("makeSpline: failed to make spline edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	return;
     }
 
@@ -513,8 +572,7 @@ void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
   /* True if either head or tail has a port on its boundary */
 #define BOUNDARY_PORT(e) ((ED_tail_port(e).side)||(ED_head_port(e).side))
 
-/* _spline_edges:
- * Basic default routine for creating edges.
+/* Basic default routine for creating edges.
  * If splines are requested, we construct the obstacles.
  * If not, or nodes overlap, the function reverts to line segments.
  * NOTE: intra-cluster edges are not constrained to
@@ -523,8 +581,7 @@ void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
  * If Nop > 1 and the spline exists, it is just copied.
  * NOTE: if edgetype = EDGETYPE_NONE, we shouldn't be here.
  */
-static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
-{
+static int spline_edges_(graph_t *g, expand_t *pmargin, int edgetype) {
     node_t *n;
     edge_t *e;
     edge_t *e0;
@@ -561,9 +618,9 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 	}
 	else {
 	    if (edgetype == EDGETYPE_ORTHO)
-		agerr(AGWARN, "the bounding boxes of some nodes touch - falling back to straight line edges\n");
+		agwarningf("the bounding boxes of some nodes touch - falling back to straight line edges\n");
 	    else 
-		agerr(AGWARN, "some nodes with margin (%.02f,%.02f) touch - falling back to straight line edges\n", pmargin->x, pmargin->y);
+		agwarningf("some nodes with margin (%.02f,%.02f) touch - falling back to straight line edges\n", pmargin->x, pmargin->y);
 	}
     }
 
@@ -583,7 +640,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     }
 #ifdef ORTHO
     else if (legal && edgetype == EDGETYPE_ORTHO) {
-	orthoEdges (g, 0);
+	orthoEdges(g, false);
 	useEdges = 1;
     }
 #endif
@@ -591,11 +648,8 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     /* spline-drawing pass */
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-/* fprintf (stderr, "%s -- %s %d\n", agnameof(agtail(e)), agnameof(aghead(e)), ED_count(e)); */
 	    node_t *head = aghead(e);
 	    if (useEdges && ED_spl(e)) {
-		add_pointf(ND_coord(n), ED_tail_port(e).p);
-		add_pointf(ND_coord(head), ED_head_port(e).p);
 		addEdgeLabels(e);
 	    } 
 	    else if (ED_count(e) == 0) continue;  /* only do representative */
@@ -652,8 +706,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     return 0;
 }
 
-/* splineEdges:
- * Main wrapper code for generating edges.
+/* Main wrapper code for generating edges.
  * Sets desired separation. 
  * Coalesces equivalent edges (edges * with the same endpoints). 
  * It then calls the edge generating function, and marks the
@@ -710,17 +763,15 @@ splineEdges(graph_t * g, int (*edgefn) (graph_t *, expand_t*, int),
     return 0;
 }
 
-/* spline_edges1:
- * Construct edges using default algorithm and given splines value.
+/* Construct edges using default algorithm and given splines value.
  * Return 0 on success.
  */
 int spline_edges1(graph_t * g, int edgetype)
 {
-    return splineEdges(g, _spline_edges, edgetype);
+  return splineEdges(g, spline_edges_, edgetype);
 }
 
-/* spline_edges0:
- * Sets the graph's aspect ratio.
+/* Sets the graph's aspect ratio.
  * Check splines attribute and construct edges using default algorithm.
  * If the splines attribute is defined but equal to "", skip edge routing.
  * 
@@ -739,7 +790,7 @@ void spline_edges0(graph_t *g, bool set_aspect) {
     if (et == EDGETYPE_NONE) return;
 #ifndef ORTHO
     if (et == EDGETYPE_ORTHO) {
-	agerr (AGWARN, "Orthogonal edges not yet supported\n");
+	agwarningf("Orthogonal edges not yet supported\n");
 	et = EDGETYPE_PLINE;
 	GD_flags(g->root) &= ~EDGETYPE_ORTHO;
 	GD_flags(g->root) |= EDGETYPE_PLINE;
@@ -763,8 +814,7 @@ shiftClusters (graph_t * g, pointf offset)
     GD_bb(g).LL.y -= offset.y;
 }
 
-/* spline_edges:
- * Compute bounding box, translate graph to origin,
+/* Compute bounding box, translate graph to origin,
  * then construct all edges.
  */
 void spline_edges(graph_t * g)
@@ -784,13 +834,11 @@ void spline_edges(graph_t * g)
     spline_edges0(g, true);
 }
 
-/* scaleEdge:
- * Scale edge by given factor.
+/* Scale edge by given factor.
  * Assume ED_spl != NULL.
  */
 static void scaleEdge(edge_t * e, double xf, double yf)
 {
-    int i, j;
     pointf *pt;
     bezier *bez;
     pointf delh, delt;
@@ -801,9 +849,9 @@ static void scaleEdge(edge_t * e, double xf, double yf)
     delt.y = POINTS_PER_INCH * (ND_pos(agtail(e))[1] * (yf - 1.0));
 
     bez = ED_spl(e)->list;
-    for (i = 0; i < ED_spl(e)->size; i++) {
+    for (size_t i = 0; i < ED_spl(e)->size; i++) {
 	pt = bez->list;
-	for (j = 0; j < bez->size; j++) {
+	for (size_t j = 0; j < bez->size; j++) {
 	    if (i == 0 && j == 0) {
 		pt->x += delt.x;
 		pt->y += delt.y;
@@ -843,9 +891,7 @@ static void scaleEdge(edge_t * e, double xf, double yf)
     }
 }
 
-/* scaleBB:
- * Scale bounding box of clusters of g by given factors.
- */
+/// scale bounding box of clusters of g by given factors
 static void scaleBB(graph_t * g, double xf, double yf)
 {
     int i;
@@ -864,20 +910,18 @@ static void scaleBB(graph_t * g, double xf, double yf)
 	scaleBB(GD_clust(g)[i], xf, yf);
 }
 
-/* translateE:
- * Translate edge by offset.
+/* Translate edge by offset.
  * Assume ED_spl(e) != NULL
  */
 static void translateE(edge_t * e, pointf offset)
 {
-    int i, j;
     pointf *pt;
     bezier *bez;
 
     bez = ED_spl(e)->list;
-    for (i = 0; i < ED_spl(e)->size; i++) {
+    for (size_t i = 0; i < ED_spl(e)->size; i++) {
 	pt = bez->list;
-	for (j = 0; j < bez->size; j++) {
+	for (size_t j = 0; j < bez->size; j++) {
 	    pt->x -= offset.x;
 	    pt->y -= offset.y;
 	    pt++;
@@ -956,8 +1000,7 @@ void neato_translate(Agraph_t * g)
     translateG(g, ll);
 }
 
-/* _neato_set_aspect;
- * Assume all bounding boxes are correct.
+/* Assume all bounding boxes are correct.
  * Return false if no transform is performed. This includes
  * the possibility that a translation was done.
  */
@@ -983,8 +1026,8 @@ static bool _neato_set_aspect(graph_t * g)
 	    /* fill is weird because both X and Y can stretch */
 	    if (GD_drawing(g)->size.x <= 0)
 		return translated;
-	    xf = (double) GD_drawing(g)->size.x / GD_bb(g).UR.x;
-	    yf = (double) GD_drawing(g)->size.y / GD_bb(g).UR.y;
+	    xf = GD_drawing(g)->size.x / GD_bb(g).UR.x;
+	    yf = GD_drawing(g)->size.y / GD_bb(g).UR.y;
 	    /* handle case where one or more dimensions is too big */
 	    if (xf < 1.0 || yf < 1.0) {
 		if (xf < yf) {
@@ -998,8 +1041,8 @@ static bool _neato_set_aspect(graph_t * g)
 	} else if (GD_drawing(g)->ratio_kind == R_EXPAND) {
 	    if (GD_drawing(g)->size.x <= 0)
 		return translated;
-	    xf = (double) GD_drawing(g)->size.x / GD_bb(g).UR.x;
-	    yf = (double) GD_drawing(g)->size.y / GD_bb(g).UR.y;
+	    xf = GD_drawing(g)->size.x / GD_bb(g).UR.x;
+	    yf = GD_drawing(g)->size.y / GD_bb(g).UR.y;
 	    if (xf > 1.0 && yf > 1.0) {
 		double scale = fmin(xf, yf);
 		xf = yf = scale;
@@ -1018,9 +1061,7 @@ static bool _neato_set_aspect(graph_t * g)
 	} else
 	    return translated;
 	if (GD_flip(g)) {
-	    double t = xf;
-	    xf = yf;
-	    yf = t;
+	    SWAP(&xf, &yf);
 	}
 
 	if (Nop > 1) {
@@ -1045,8 +1086,7 @@ static bool _neato_set_aspect(graph_t * g)
 	return false;
 }
 
-/* neato_set_aspect:
- * Sets aspect ratio if necessary; real work done in _neato_set_aspect;
+/* Sets aspect ratio if necessary; real work done in _neato_set_aspect;
  * This also copies the internal layout coordinates (ND_pos) to the 
  * external ones (ND_coord).
  *
